#ifndef GLCD_KS0108_H
#define	GLCD_KS0108_H
//------------------------------------------------------------------------------
#include "charset.h"
#include "picture.h"
#include "GLCD_Pins.h"
#include "GLCD_Prototypes.h"
#include "../MCP23017_PortExpander/MCP23017_Expander.h"
//------------------------------------------------------------------------------
void GLCD_chip(char chip)
{
  switch(chip)
  {
    case 1: CS = 0x04; break;  //Selects Chip 1. NB: Both are Active LOW!
    case 2: CS = 0x08; break;  //Selects Chip 2
    case 3: CS = 0x00; break;  //Selects Both
  }
}
//------------------------------------------------------------------------------
void GLCD_command(char command, char chip)
{
  GLCD_chip(chip);
  RS = 0;                       //Instruction or Command Register
  char controlData = CS + RS;
  
  EXPANDER_write(OLATB, command);
  GLCD_latch(controlData);
}
//------------------------------------------------------------------------------
void GLCD_data(char data, char chip)
{
  GLCD_chip(chip);
  RS = 1;                       //Data Register
  char controlData = CS + RS;
  
  EXPANDER_write(OLATB, data);
  GLCD_latch(controlData);
}
//------------------------------------------------------------------------------
void GLCD_column(char col)
{
  column = col;                      //column is a global variable that keeps track of column infomation
  if( col < pgWidth )
  {
    col = 0x40 | col;                //0x40 represents Column 0
    GLCD_command(col,1);
  }
  else
      if( col < glcdWidth )
      {
        col = 0x40 | (col - pgWidth);
        GLCD_command(col,2);
      }
}
//------------------------------------------------------------------------------
void GLCD_page(char pg)
{
  pg = page[pg];
  GLCD_command(pg,3);
}
//------------------------------------------------------------------------------
void GLCD_startline(char line)
{
  line = 0xC0 | line;	   //0xC0 represents Line 0
  GLCD_command(line,3);    //Write to Chip 1 and 2
}
//------------------------------------------------------------------------------
void GLCD_bitmap(unsigned char *bitmap, int elements)
{
  for(int e = 0 ; e < elements ; e++)
  {
    if( column < pgWidth )
    {
      GLCD_data(bitmap[e],1);       // Write to Chip 1
    }
    else
        if( column < glcdWidth )
        {
          GLCD_column(column);
          GLCD_data(bitmap[e],2);    //Write to Chip 2
        }
    column++;
  }
}
//------------------------------------------------------------------------------
void GLCD_font1(int pg,int col,unsigned char *text)
{
  GLCD_page(pg);
  GLCD_column(col);

  while( *text )
  {
    int row = *text++;
    GLCD_bitmap( &Character8x8[row*8] , 8 ); //Pass Address(with index as a multiple of 8) with the array size
  }
}
//------------------------------------------------------------------------------
void GLCD_font2(int pg,int col,unsigned char *text)
{
  GLCD_page(pg);
  GLCD_column(col);
 
  while( *text  )
  {
    int row = *text++ - 32 ;
    GLCD_bitmap( &font5x7[row*5] , 5);
  }
}
//------------------------------------------------------------------------------
void GLCD_image(unsigned char *image)
{
  for(int pg = 0; pg < pgLen ;pg++)
  {
    GLCD_page(pg);
    GLCD_column(0);
    GLCD_bitmap( &image[pg*glcdWidth] , glcdWidth );
  }
}
//------------------------------------------------------------------------------
void GLCD_fill(char fill)
{
  for(int pg = 0; pg < pgLen; pg++)
  {
    GLCD_page(pg);
    GLCD_column(0);
    for(int col = 0; col < glcdWidth; col++)
    {
      GLCD_bitmap(&fill,1);
    }
  } 
}
//------------------------------------------------------------------------------
void GLCD_init()
{    
  GLCD_command(ON,1);
  GLCD_fill(0x00);
}
//------------------------------------------------------------------------------
void GLCD_latch(char controlData)
{
  EXPANDER_write(OLATA, (controlData | EN_HIGH));
  EXPANDER_write(OLATA, (controlData & EN_LOW));
}
//------------------------------------------------------------------------------
#endif